#!/usr/bin/python3

"""
Syntax: {SCRIPTNAME} ifile [perline] [ofile] [nobuiltin]

Convert {ifile} into bash commands that reconstitute the original file.
Commands may be pasted in order, no internal state is kept.

{perline} bytes will be written per printf() call, default 32.
{ofile} will specify reconstituted output filename, default ifile.out.

"""

import sys


def help(SCRIPTNAME):
        print(f"Syntax: {SCRIPTNAME} ifile [perline] [ofile] [nobuiltin]")
        print()
        print("""This script converts an input file "ifile" into a bash script that can be executed to recreate the file original file.

The output script does not maintain internal state and can be pasted into or executed by the host as individual lines.
It relies solely on bash's built in functionality and does not depend on any external binaries.

Line length can be adjusted via the "perline" parameter.
        Special: if perline is set to -1, the script will output lines as short as possible. (18 characters minimum, depending on path)
        (This removes the "builtin" command protection, sets perline to 1 and shrinks your output filename but preserves the path)

The output file and path can be specified via the "ofile" parameter.

The "nobuiltin" parameter, if specified, will remove the "builtin" prefix, which will allow the shell to use /bin/printf.
""")
              
        

        sys.exit(1)

def main(argv=None):
        global SCRIPTNAME
        micromode = False
        BUILTIN_GUARD = True
        DEFAULTBYTES = 32

        # handle command line arguments
        try:
                SCRIPTNAME = argv[0]
                ifile = argv[1]

                perline = DEFAULTBYTES
                if len(argv) > 2:
                        perline = int(argv[2])

                        if perline == -1:
                                micromode = True

                ofile = ifile.split("/")[-1] + ".out"
                if len(argv) > 3:
                        ofile = argv[3]

                if len(argv) > 4:
                        if argv[4] == "nobuiltin":
                                BUILTIN_GUARD = False

        except IndexError:
                help(argv[0])
                
        with open(ifile, "rb") as f:

                byte = f.read(1)
                buffer = ""
                counter = 1

                if micromode:
                        perline = 1
                        ofile = ofile.split("/")
                        ofile[-1] = ofile[-1][0]
                        ofile = "/".join(ofile)
                        BUILTIN_GUARD = False

                # probably not needed, but just in case
                print(f"#!/usr/bin/bash")

                # clear out the destination file
                print(f">{ofile}")

                #output bytes from ifile as printf commands
                while byte:
                        buffer += (hex(ord(byte))).replace("0x","\\x")

                        if counter >= perline:
                                if BUILTIN_GUARD:
                                        print(f"{{builtin,printf,'{buffer}'}}>>{ofile}")
                                else:
                                        print(f"{{printf,'{buffer}'}}>>{ofile}")
                                buffer = ""
                                counter = 0

                        counter += 1
                        byte = f.read(1)
                if BUILTIN_GUARD:
                        print(f"{{builtin,printf,'{buffer}'}}>>{ofile}")
                else:
                        print(f"{{printf,'{buffer}'}}>>{ofile}")

if __name__ == '__main__':
        sys.exit(main(sys.argv))
